local _G = _G

local BRFForwarder = _G.BRFForwarder

-- addon constants
local MODNAME   = "BRFForwarder"

LibStub:GetLibrary("AceComm-3.0"):Embed(BRFForwarder)
LibStub:GetLibrary("AceSerializer-3.0"):Embed(BRFForwarder)

-- local functions
local math_ceil  = math.ceil
local pairs      = pairs
local next       = next
local time       = time
local math_rnd   = math.random

-- send info data to new owner at some point within this time interval
local INITINTERVAL     =  10

function BRFForwarder:SetupCommunication()
	-- init pseudo-random generator
	-- NOTE: not available?
	-- math.randomseed(time())

	-- do it as field so owner changes during timeout don't cause misdirected notifications
	self.pending_send = {}
	
	-- set up actual communication
	self:ActivateCommunication(true)	
end

function BRFForwarder:ShutdownCommunication()
	self:ActivateCommunication(false)
end

function BRFForwarder:ActivateCommunication(activate)
	self.comm_activate = activate
	
	self:PrepareCommunication()
end

function BRFForwarder:PrepareCommunication()
	self:Debug("PrepareCommunication")

	if self.comm_activate == true then
		if not self.comm_ready then
			self:Debug("PrepareCommunication: setup communication")

			self.comm_ready = true
			self:RegisterComm(MODNAME, "OnCommReceive")	
		end
	else
		if self.comm_ready == true then
			self:Debug("PrepareCommunication: shutdown communication")

			self:UnregisterComm(MODNAME)
			self.comm_ready = false
		end
	end
end

function BRFForwarder:CancelPendingNotifications(target)
	if self.pending_send[target] then
		self:CancelTimer(self.pending_send[target])
		self.pending_send[target] = nil
	end
end

-- update owner about our config
function BRFForwarder:NotifyOwner(do_delay)
	self:Debug("NotifyOwner")
	
	if not self.owner then
		return
	end

	if self:IsOwner() then
		return
	end
	
	if do_delay then
		if not self.pending_send[self.owner] then
			-- try to soften the possible message burst on receiver side when owner just changed
			-- all users in town will update their status after new owner has been selected
			self.pending_send[self.owner] = self:ScheduleTimer("SendInfoData", math_rnd()*INITINTERVAL, self.owner)
		end
	else		
		self:SendInfoData(self.owner)
	end	
end

-- communication

-- update ready info
function BRFForwarder:SendInfoData(receiver)
	self:Debug("SendInfoData to " .. tostring(receiver))
			
	if not receiver then
		self:Debug("SendInfoData: no receiver")
		return
	end

	if receiver == self.PlayerName then
		self:Debug("SendInfoData: invalid receiver")
		return
	end

	self:CancelPendingNotifications(receiver)
	
	local data = self:IsReadyToForward()
	
	self:SendData(receiver, data)
end

-- send the data
function BRFForwarder:SendData(receiver, data, isSerialized, force)
	if receiver == self.PlayerName then
		self:Debug("SendData to self rejected")
		return
	end
	
	local serializeddata = data
	
	if not isSerialized then
		serializeddata = self:Serialize(data)
	end
	
	self:Debug("SendData to " .. tostring(receiver))
	self:SendCommMessage(MODNAME, serializeddata, "WHISPER", receiver)
end

-- callback for received messages
function BRFForwarder:OnCommReceive(prefix, msgs, distribution, sender)
	self:Debug("OnCommReceive")
	-- shouldn't happen, which means we got to test for it
	if not self.comm_ready then
		return
	end
	
	if not self:IsOwner() then
		self:Debug("OnCommReceive: we are not owner - dropping package")
		return
	end
	
	-- deserialize
	self:Debug("OnCommReceive: deserialize data (sent on distri chan " .. tostring(distribution) .. ")")
	local success, data = self:Deserialize(msgs)
	
	if not success then
		self:Debug("OnCommReceive: data corrupt")
		return
	end
	
	self:Debug("OnCommReceive: data " .. tostring(data))
	
	if not sender then
		-- huh?
		self:Debug("OnCommReceive: no sender")
		return
	end
	
	-- process message
	self:HandleInfoData(data, sender)
end

function BRFForwarder:HandleInfoData(ready, sender)
	self:Debug("HandleInfoData: sender " .. tostring(sender) .. " ready " .. tostring (ready))

	if not sender then
		self:Debug("HandleInfoData: no sender")
		return
	end
	
	if ready then
		self:Debug("HandleInfoData: SetCandidate " .. tostring(sender))
		self:SetCandidate(sender)
		
		if self:IsOwner() and not self:IsReadyToForward() then
			self:Debug("HandleInfoData: TryTransferOwnership")
			self:TryTransferOwnership()
		end
	else
		self:Debug("HandleInfoData: RemoveCandidate " .. tostring(sender))
		self:RemoveCandidate(sender)
	end
	
end
