local _G = _G

local BrokerRaidFinder = _G.BrokerRaidFinder

-- addon constants
local MODNAME   = "BrokerRaidFinder"
local FULLNAME  = "Broker: Raid Finder"

-- get translations
local L             = LibStub:GetLibrary("AceLocale-3.0"):GetLocale( MODNAME )
local LibBabbleZone = LibStub:GetLibrary("LibBabble-Zone-3.0"):GetLookupTable()

-- tooltip library
local QT = LibStub:GetLibrary("LibQTip-1.0")

-- coloring tools
local Crayon	= LibStub:GetLibrary("LibCrayon-3.0")

-- local functions
local string_format     = string.format
local pairs             = pairs
local next              = next
local floor             = floor

-- utilities
local function format_time(stamp)
	local days    = floor(stamp/86400)
	local hours   = floor((stamp - days * 86400) / 3600)
	local minutes = floor((stamp - days * 86400 - hours * 3600) / 60)
	local seconds = floor((stamp - days * 86400 - hours * 3600 - minutes * 60))

	return string_format("%02d:%02d:%02d", hours, minutes, seconds)
end

-- shortcuts
local function Red( text )      return BrokerRaidFinder:Colorize("Red",      text) end
local function Green( text )    return BrokerRaidFinder:Colorize("Green",    text) end
local function Yellow( text )   return BrokerRaidFinder:Colorize("Yellow",   text) end
local function White( text )    return BrokerRaidFinder:Colorize("White",    text) end
local function Brownish( text ) return BrokerRaidFinder:Colorize("Brownish", text) end
local function Orange( text )   return BrokerRaidFinder:Colorize("Orange",   text) end
local function Blueish( text )  return BrokerRaidFinder:Colorize("Blueish",  text) end
local function GrayOut( text )  return BrokerRaidFinder:Colorize("GrayOut",  text) end

local function BRF_ToggleInstance(cell, instance, button)
	BrokerRaidFinder:ToggleInstance(instance)
	BrokerRaidFinder:DrawTooltip()
	tooltip:Show()
end

local tooltip

function BrokerRaidFinder:CreateTooltip( obj )
	tooltip = QT:Acquire( MODNAME.."TT", 3 )
	tooltip:Hide()
	tooltip:Clear()
	tooltip:SetScale(1)
		
	BrokerRaidFinder:DrawTooltip()

	tooltip:SetAutoHideDelay( .1, obj )
	tooltip:EnableMouse()
	tooltip:SmartAnchorTo( obj )
	tooltip:Show()
end

function BrokerRaidFinder:RemoveTooltip()
	if tooltip then
		tooltip:Hide()
		QT:Release(tooltip)
		tooltip = nil
	end
end

-- tooltip
function BrokerRaidFinder:DrawTooltip()
	tooltip:Hide()
	tooltip:Clear()

	local now = time()
	
	-- addon header
	local lineNum = tooltip:AddHeader( " " )
	tooltip:SetCell( lineNum, 1, White( FULLNAME ), "CENTER", tooltip:GetColumnCount() )
	
	local ctrlKeyPressed = IsControlKeyDown()
	
	-- monitored raids
	if ctrlKeyPressed or self.db.profile.monitoringActive then
		-- for all extensions
		for extension, instances in pairs(brf_Instances) do
			local header_present = false
			
			-- for all instances
			for _, instance in pairs(instances) do
				local monitored = self:IsMonitored(instance)
				
				if ctrlKeyPressed or (self:IsMonitored(instance) and not self:ExcludeAsSaved(instance)) then 
					local saved = self:IsSaved(instance)
				
					if not header_present then
						-- set extension name as header
						tooltip:AddLine( " " )
						lineNum = tooltip:AddLine( " " )
						tooltip:SetCell( lineNum, 1, Orange( L[brf_Extensions[extension]] ), "LEFT", tooltip:GetColumnCount() )
						
						header_present = true
					end

					lineNum = tooltip:AddLine( " " )
					
					if self.matches_latest[instance] then
						-- instance
						if saved then
							tooltip:SetCell( lineNum, 1, Red( LibBabbleZone[instance] ) .. " ", "LEFT")
						else
							tooltip:SetCell( lineNum, 1, Green( LibBabbleZone[instance] ) .. " ", "LEFT")
						end

						local match = self.matches_latest[instance]
						
						-- last player posting
						tooltip:SetCell( lineNum, 2, self:ColorizeChar( match.player ) .. White(" (of " .. tostring(match.players) .. ") ") , "LEFT")
						
						-- time since posting
						local timestamp = now - match.timestamp
						local timestring = ""
						if Crayon then
							local timeframe = self.db.profile.timeFrame * 60
							timestring = "|cff"..Crayon:GetThresholdHexColor(timestamp, timeframe, timeframe * 0.75, timeframe * 0.5, timeframe * 0.25, 1) .. format_time(timestamp) .. "|r"
						else
							timestring = format_time(timestamp)
						end				
						
						tooltip:SetCell( lineNum, 3, timestring, "RIGHT")
					else
						-- instance
						if saved then
							tooltip:SetCell( lineNum, 1, Red( LibBabbleZone[instance] ) .. " ", "LEFT")
						elseif monitored then
							tooltip:SetCell( lineNum, 1, Blueish( LibBabbleZone[instance] ) .. " ", "LEFT")
						else
							tooltip:SetCell( lineNum, 1, GrayOut( LibBabbleZone[instance] ) .. " ", "LEFT")
						end
						
						-- no player
						tooltip:SetCell( lineNum, 2, GrayOut( "--" ), "LEFT")
						
						-- no time
						tooltip:SetCell( lineNum, 3, GrayOut( "--" ), "RIGHT")
					end
					
					tooltip:SetCellScript( lineNum, 1, "OnMouseDown", BRF_ToggleInstance, instance)					
				end			
			end	
		end
		
		if self.server then
			tooltip:AddLine( " " )
			lineNum = tooltip:AddLine( " " )
			tooltip:SetCell( lineNum, 1, Green( L["Remote client is monitoring."] ), "LEFT", tooltip:GetColumnCount() )
		end
	else
		lineNum = tooltip:AddLine( " " )
		tooltip:SetCell( lineNum, 1, Red( L["Monitoring is switched off."] ), "LEFT", tooltip:GetColumnCount() )
	end
	
	-- plugins
	for name, plugin in pairs(self.plugins) do
		if self:ShowPluginTooltip(name) and plugin:IsActive() then
			local messages = plugin:GetTooltipMessages() or {}
			
			if #messages > 0 then
				tooltip:AddLine( " " )
				lineNum = tooltip:AddLine( " " )
				tooltip:SetCell( lineNum, 1, Brownish( plugin:GetPluginName() .. " " .. L["Plugin"] .. ":"), "LEFT", tooltip:GetColumnCount() )
			end
			
			for idx, msg in ipairs(messages) do
				lineNum = tooltip:AddLine( " " )
				tooltip:SetCell( lineNum, 1, msg, "LEFT", tooltip:GetColumnCount() )
			end
		end
	end
	
	-- hint to show options
	if not self.db.profile.hideHint then
		tooltip:AddLine( " " )
		lineNum = tooltip:AddLine( " " )
		tooltip:SetCell( lineNum, 1, Brownish( L["Left-Click"] .. ":") .. Yellow( " " .. L["Show logging window."] ), "LEFT", tooltip:GetColumnCount() )
		lineNum = tooltip:AddLine( " " )
		tooltip:SetCell( lineNum, 1, Brownish( L["Alt-Click"] .. ":") .. Yellow( " " .. L["Enable/disable monitoring."] ), "LEFT", tooltip:GetColumnCount() )
		lineNum = tooltip:AddLine( " " )
		tooltip:SetCell( lineNum, 1, Brownish( L["Right-Click"] .. ":") .. Yellow( " " .. L["Open option menu."] ), "LEFT", tooltip:GetColumnCount() )
		lineNum = tooltip:AddLine( " " )
		tooltip:SetCell( lineNum, 1, Brownish( L["Left-Click Tip"] .. ":") .. Yellow( " " .. L["Click on instance name to toggle monitoring."] ), "LEFT", tooltip:GetColumnCount() )
		lineNum = tooltip:AddLine( " " )
		tooltip:SetCell( lineNum, 1, Brownish( L["Ctrl-Hover Tip"] .. ":") .. Yellow( " " .. L["Press Ctrl while opening tooltip shows all instances."] ), "LEFT", tooltip:GetColumnCount() )
	end
	
end

